package net.wanji.web.service.scheme.impl;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import net.wanji.common.dto.CrossIdDTO;
import net.wanji.web.common.constant.ControlModeConst;
import net.wanji.web.common.exception.NoLanesException;
import net.wanji.web.common.exception.PhaseNoNotUniqueException;
import net.wanji.web.dto.LaneIdDTO;
import net.wanji.web.dto.LightIdDTO;
import net.wanji.web.dto.SaveSchemeConfigDTO;
import net.wanji.web.mapper.scheme.CrossAccompanyPhaseMapper;
import net.wanji.web.mapper.scheme.CrossLaneLightsMapper;
import net.wanji.web.mapper.scheme.CrossLightsMapper;
import net.wanji.web.mapper.scheme.CrossPhaseLightsMapper;
import net.wanji.web.mapper.scheme.CrossPhaseMapper;
import net.wanji.web.mapper.scheme.CrossSchemeMapper;
import net.wanji.web.mapper.scheme.LaneInfoMapper;
import net.wanji.web.po.scheme.CrossAccompanyPhasePO;
import net.wanji.web.po.scheme.CrossLaneLightsPO;
import net.wanji.web.po.scheme.CrossLightsPO;
import net.wanji.web.po.scheme.CrossPhaseLightsPO;
import net.wanji.web.po.scheme.CrossPhasePO;
import net.wanji.web.po.scheme.CrossSchemePO;
import net.wanji.web.po.scheme.LaneInfoPO;
import net.wanji.web.service.scheme.SchemeConfigService;
import net.wanji.web.vo.scheme.LaneIdsVO;
import net.wanji.web.vo.scheme.LightIdVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.TreeSet;
import java.util.stream.Collectors;

/**
 * @author Kent HAN
 * @date 2023/1/5 10:27
 */
@Service
public class SchemeConfigServiceImpl implements SchemeConfigService {
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final LaneInfoMapper laneInfoMapper;
    private final CrossLaneLightsMapper crossLaneLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossAccompanyPhaseMapper crossAccompanyPhaseMapper;

    public SchemeConfigServiceImpl(CrossSchemeMapper crossSchemeMapper, CrossPhaseMapper crossPhaseMapper,
                                   LaneInfoMapper laneInfoMapper, CrossLaneLightsMapper crossLaneLightsMapper,
                                   CrossPhaseLightsMapper crossPhaseLightsMapper, CrossLightsMapper crossLightsMapper, CrossAccompanyPhaseMapper crossAccompanyPhaseMapper) {
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossPhaseMapper = crossPhaseMapper;
        this.laneInfoMapper = laneInfoMapper;
        this.crossLaneLightsMapper = crossLaneLightsMapper;
        this.crossPhaseLightsMapper = crossPhaseLightsMapper;
        this.crossLightsMapper = crossLightsMapper;
        this.crossAccompanyPhaseMapper = crossAccompanyPhaseMapper;
    }

    @Override
    @Transactional
    public void saveSchemeConfig(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        Integer timeType = saveSchemeConfigDTO.getTimeType();
        if (timeType == 2) { //环式
            handleRingTime(saveSchemeConfigDTO);
        } else if (timeType == 1) { //阶段式
            // TODO: 2023/1/6 处理阶段式
        }
    }

    @Override
    public LightIdVO getLightByLane(LaneIdDTO laneIdDTO) {
        String crossId = laneIdDTO.getCrossId();
        String laneId = laneIdDTO.getLaneId();
        Integer lightId = crossLaneLightsMapper.selectLightsIdByLaneId(crossId, laneId);
        LightIdVO lightIdVO = new LightIdVO();
        lightIdVO.setCrossId(crossId);
        lightIdVO.setLightId(lightId);
        return lightIdVO;
    }

    @Override
    public LaneIdsVO getLanesByLight(LightIdDTO lightIdDTO) {
        String crossId = lightIdDTO.getCrossId();
        LaneIdsVO laneIdsVO = new LaneIdsVO();
        laneIdsVO.setCrossId(crossId);

        Integer lightId = lightIdDTO.getLightId();
        List<CrossLaneLightsPO> crossLaneLightsPOList = crossLaneLightsMapper.selectLanesByLightId(crossId, lightId);
        List<String> laneIds = crossLaneLightsPOList.stream()
                .map(CrossLaneLightsPO::getLaneId)
                .collect(Collectors.toList());
        if (CollectionUtil.isEmpty(laneIds)) {
            throw new NoLanesException("灯组没有对应的车道信息，请先完成灯组配置");
        }
        List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectByLaneIds(laneIds);
        laneIdsVO.setLanes(laneInfoPOList);

        return laneIdsVO;
    }

    @Override
    @Transactional
    public SaveSchemeConfigDTO listSchemeConfig(CrossIdDTO crossIdDTO) {
        String crossId = crossIdDTO.getCrossId();
        SaveSchemeConfigDTO saveSchemeConfigDTO = new SaveSchemeConfigDTO();
        saveSchemeConfigDTO.setCrossId(crossId);
        // 构造方案列表
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = buildPhaseSchemeList(crossId);
        saveSchemeConfigDTO.setPhaseSchemeList(phaseSchemeList);

        // todo 暂时只有环式
        saveSchemeConfigDTO.setTimeType(2);
        return saveSchemeConfigDTO;
    }

    /**
     * 构造方案列表
     *
     * @author Kent HAN
     * @date 2023/1/12 9:23
     * @param crossId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.PhaseScheme>
     */
    private List<SaveSchemeConfigDTO.PhaseScheme> buildPhaseSchemeList(String crossId) {
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = new ArrayList<>();
        List<CrossSchemePO> crossSchemePOList = crossSchemeMapper.selectByCrossId(crossId);
        for (CrossSchemePO crossSchemePO : crossSchemePOList) {
            SaveSchemeConfigDTO.PhaseScheme phaseScheme = new SaveSchemeConfigDTO.PhaseScheme();
            phaseScheme.setSchemeNo(crossSchemePO.getSchemeNo());
            phaseScheme.setName(crossSchemePO.getName());
            phaseScheme.setSchemeId(crossSchemePO.getId());
            // 获取协调相位号
            Integer coordPhaseId = crossSchemePO.getCoordPhase();
            if (coordPhaseId != null) {
                CrossPhasePO crossPhasePO = crossPhaseMapper.selectById(coordPhaseId);
                if (crossPhasePO != null) {
                    phaseScheme.setCoordPhaseNo(crossPhasePO.getPhaseNo());
                }
            }
            phaseScheme.setCycle(crossSchemePO.getCycle());
            phaseScheme.setOffset(crossSchemePO.getOffset());
            phaseScheme.setStatus(crossSchemePO.getStatus());
            // 构造环列表
            String schemeNo = phaseScheme.getSchemeNo();
            Integer schemeId = crossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
            List<SaveSchemeConfigDTO.RingListElement> ringList = buildRingList(crossId, schemeId);
            phaseScheme.setRingList(ringList);
            // 构造特殊控制列表
            List<SaveSchemeConfigDTO.PhaseListElement> specialControlList = buildSpecialControlList(crossId, schemeId);
            phaseScheme.setSpecialControlModeList(specialControlList);

            phaseSchemeList.add(phaseScheme);
        }
        return phaseSchemeList;
    }

    /**
     * 构造特殊控制列表
     *
     * @author Kent HAN
     * @date 2023/1/12 17:53
     * @param crossId
     * @param schemeId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.PhaseListElement>
     */
    private List<SaveSchemeConfigDTO.PhaseListElement> buildSpecialControlList(String crossId, Integer schemeId) {
        List<SaveSchemeConfigDTO.PhaseListElement> phaseListElementList = new ArrayList<>();

        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByCrossIdAndSchemeId(crossId, schemeId);
        for (CrossPhasePO crossPhasePO : crossPhasePOList) {
            SaveSchemeConfigDTO.PhaseListElement phaseListElement = new SaveSchemeConfigDTO.PhaseListElement();
            Integer controlMode = crossPhasePO.getControlMode();
            if (Objects.equals(controlMode, ControlModeConst.IS_GREEN)
                    || Objects.equals(controlMode, ControlModeConst.IS_RED)
                    || Objects.equals(controlMode, ControlModeConst.GREEN_FLASH)
                    || Objects.equals(controlMode, ControlModeConst.YELLOW_FLASH)) {
                String phaseNo = crossPhasePO.getPhaseNo();
                phaseListElement.setPhaseNo(phaseNo);
                phaseListElement.setPhaseType(crossPhasePO.getPhaseType());
                phaseListElement.setMinGreenTime(crossPhasePO.getMinGreenTime());
                phaseListElement.setMaxGreenTime(crossPhasePO.getMaxGreenTime());
                phaseListElement.setGreenTime(crossPhasePO.getGreenTime());
                phaseListElement.setGreenFlashTime(crossPhasePO.getGreenFlashTime());
                phaseListElement.setYellowFlashTime(crossPhasePO.getYellowFlashTime());
                phaseListElement.setRedFlashTime(crossPhasePO.getRedFlashTime());
                phaseListElement.setYellowTime(crossPhasePO.getYellowTime());
                phaseListElement.setRedTime(crossPhasePO.getRedTime());
                phaseListElement.setYellowFlash(crossPhasePO.getYellowFlashTime());
                if (controlMode.equals(ControlModeConst.IS_RED)){
                    phaseListElement.setIsRed(1);
                } else if (controlMode.equals(ControlModeConst.IS_GREEN)) {
                    phaseListElement.setIsGreen(1);
                } else if (controlMode.equals(ControlModeConst.GREEN_FLASH)) {
                    phaseListElement.setGreenFlash(1);
                } else if (controlMode.equals(ControlModeConst.YELLOW_FLASH)) {
                    phaseListElement.setYellowFlash(1);
                }
                phaseListElement.setAccompanyPhaseNoList(new ArrayList<>());
                Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndCoordPhaseNo(crossId, schemeId, phaseNo);
                List<SaveSchemeConfigDTO.DirListElement> dirList = buildDirList(crossId, phaseId);
                phaseListElement.setDirList(dirList);

                phaseListElementList.add(phaseListElement);
            }
        }
        return phaseListElementList;
    }

    /**
     * 构造环列表
     *
     * @author Kent HAN
     * @date 2023/1/12 9:28
     * @param crossId
     * @param schemeId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.RingListElement>
     */
    private List<SaveSchemeConfigDTO.RingListElement> buildRingList(String crossId, Integer schemeId) {
        List<SaveSchemeConfigDTO.RingListElement> ringlist = new ArrayList<>();
        List<Integer> ringNoList = crossPhaseMapper.selectRingNumbersByCrossIdAndSchemeId(crossId, schemeId);
        for (Integer ringNo : ringNoList) {
            SaveSchemeConfigDTO.RingListElement ringListElement = new SaveSchemeConfigDTO.RingListElement();
            if (ringNo != 0) {
                ringListElement.setRingNo(ringNo);
                // 构造组列表
                List<SaveSchemeConfigDTO.GroupListElement> groupList = buildGroupList(crossId, schemeId, ringNo);
                ringListElement.setGroupList(groupList);
                ringlist.add(ringListElement);
            }
        }
        return ringlist;
    }

    /**
     * 构造组列表
     *
     * @author Kent HAN
     * @date 2023/1/12 9:42
     * @param crossId
     * @param schemeId
     * @param ringNo
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.GroupListElement>
     */
    private List<SaveSchemeConfigDTO.GroupListElement> buildGroupList(
            String crossId, Integer schemeId, Integer ringNo) {
        List<SaveSchemeConfigDTO.GroupListElement> groupList = new ArrayList<>();
        List<Integer> groupNumbers = crossPhaseMapper.selectGroupNumbers(crossId, schemeId, ringNo);
        for (Integer groupNumber : groupNumbers) {
            SaveSchemeConfigDTO.GroupListElement groupListElement = new SaveSchemeConfigDTO.GroupListElement();
            groupListElement.setGroupNo(groupNumber);
            // 构造相位列表
            List<SaveSchemeConfigDTO.PhaseListElement> phaseList =
                    buildPhaseList(crossId, schemeId, ringNo, groupNumber);
            groupListElement.setPhaseList(phaseList);

            groupList.add(groupListElement);
        }
        return groupList;
    }

    /**
     * 构造相位列表
     *
     * @author Kent HAN
     * @date 2023/1/12 14:58
     * @param crossId
     * @param schemeId
     * @param ringNo
     * @param groupNumber
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.PhaseListElement>
     */
    private List<SaveSchemeConfigDTO.PhaseListElement> buildPhaseList(
            String crossId, Integer schemeId, Integer ringNo, Integer groupNumber) {
        List<SaveSchemeConfigDTO.PhaseListElement> phaseList = new ArrayList<>();
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByGroupNo(crossId, schemeId, ringNo, groupNumber);
        for (CrossPhasePO crossPhasePO : crossPhasePOList) {
            SaveSchemeConfigDTO.PhaseListElement phaseListElement = new SaveSchemeConfigDTO.PhaseListElement();
            phaseListElement.setPhaseId(crossPhasePO.getId());
            String phaseNo = crossPhasePO.getPhaseNo();
            phaseListElement.setPhaseNo(phaseNo);
            Integer phaseType = crossPhasePO.getPhaseType();
            phaseListElement.setPhaseType(phaseType);
            phaseListElement.setMinGreenTime(crossPhasePO.getMinGreenTime());
            phaseListElement.setMaxGreenTime(crossPhasePO.getMaxGreenTime());
            phaseListElement.setGreenTime(crossPhasePO.getGreenTime());
            phaseListElement.setGreenFlashTime(crossPhasePO.getGreenFlashTime());
            phaseListElement.setYellowFlashTime(crossPhasePO.getYellowFlashTime());
            phaseListElement.setRedFlashTime(crossPhasePO.getRedFlashTime());
            phaseListElement.setYellowTime(crossPhasePO.getYellowTime());
            phaseListElement.setRedTime(crossPhasePO.getRedTime());
            // 特殊控制
            Integer controlMode = crossPhasePO.getControlMode();
            if (Objects.equals(controlMode, ControlModeConst.YELLOW_FLASH)) {
                phaseListElement.setYellowFlash(1);
            } else if (Objects.equals(controlMode, ControlModeConst.IS_RED)) {
                phaseListElement.setIsRed(1);
            } else if (Objects.equals(controlMode, ControlModeConst.IS_GREEN)) {
                phaseListElement.setIsGreen(1);
            } else if (Objects.equals(controlMode, ControlModeConst.GREEN_FLASH)) {
                phaseListElement.setGreenFlash(1);
            }
            // 伴随相位的母相位列表
            Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndCoordPhaseNo(crossId, schemeId, phaseNo);
            if (phaseType == 3) {
                List<String> superPhaseNoList = new ArrayList<>();
                List<Integer> superIdList = crossAccompanyPhaseMapper.selectByPhaseId(phaseId);
                List<CrossPhasePO> superList = crossPhaseMapper.selectByIds(superIdList);
                for (CrossPhasePO phasePO : superList) {
                    superPhaseNoList.add(phasePO.getPhaseNo());
                }
                phaseListElement.setAccompanyPhaseNoList(superPhaseNoList);
            }
            // 构造方向列表
            List<SaveSchemeConfigDTO.DirListElement> dirList = buildDirList(crossId, phaseId);
            phaseListElement.setDirList(dirList);

            phaseList.add(phaseListElement);
        }
        return phaseList;
    }

    /**
     * 构造方向列表
     *
     * @author Kent HAN
     * @date 2023/1/12 15:02
     * @param crossId
     * @param phaseId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.DirListElement>
     */
    private List<SaveSchemeConfigDTO.DirListElement> buildDirList(String crossId, Integer phaseId) {
        List<SaveSchemeConfigDTO.DirListElement> dirList = new ArrayList<>();
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsMapper.selectByPhaseId(phaseId);
        Set<Integer> dirSet = new HashSet<>();
        for (CrossPhaseLightsPO crossPhaseLightsPO : crossPhaseLightsPOList) {
            Integer lightsId = crossPhaseLightsPO.getLightsId();
            CrossLightsPO crossLightsPO = crossLightsMapper.selectById(lightsId);
            if (crossLightsPO != null) {
                Integer dir = crossLightsPO.getDir();
                dirSet.add(dir);
            }
        }
        for (Integer dir : dirSet) {
            SaveSchemeConfigDTO.DirListElement dirListElement = new SaveSchemeConfigDTO.DirListElement();
            dirListElement.setDir(dir);
            // 是否有行人相位
            List<CrossLightsPO> crossLightsPOList = crossLightsMapper.selectByCrossIdAndDir(crossId, dir);
            for (CrossLightsPO crossLightsPO : crossLightsPOList) {
                Integer type = crossLightsPO.getType();
                if (type >= 20 && type < 30) {
                    Integer lightsId = crossLightsPO.getId();
                    if (hasLightsId(lightsId)) {
                        dirListElement.setHasPersonPhase(1);
                    }
                }
            }
            // 构造车道列表
            List<SaveSchemeConfigDTO.LaneListElement> laneList = buildLaneList(crossId, dir, phaseId);
            dirListElement.setLaneList(laneList);

            dirList.add(dirListElement);
        }
        return dirList;
    }

    /**
     * 构造车道列表
     *
     * @author Kent HAN
     * @date 2023/1/12 15:02
     * @param crossId
     * @param dir
     * @param phaseId
     * @return java.util.List<net.wanji.web.dto.SaveSchemeConfigDTO.LaneListElement>
     */
    private List<SaveSchemeConfigDTO.LaneListElement> buildLaneList(String crossId, Integer dir, Integer phaseId) {
        List<SaveSchemeConfigDTO.LaneListElement> laneList = new ArrayList<>();
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsMapper.selectByPhaseId(phaseId);
        List<Integer> lightsIds = crossPhaseLightsPOList.stream()
                .map(CrossPhaseLightsPO::getLightsId)
                .collect(Collectors.toList());
        List<CrossLightsPO> crossLightsPOList = crossLightsMapper.selectByLightsIdsAndDir(lightsIds, dir);
        for (CrossLightsPO crossLightsPO : crossLightsPOList) {
            Integer type = crossLightsPO.getType();
            if (type < 20 || type >= 30) { // 非行人灯
                Integer lightsId = crossLightsPO.getId();
                List<CrossLaneLightsPO> crossLaneLightsPOList = crossLaneLightsMapper.selectByLightsId(lightsId);
                List<String> laneIds = crossLaneLightsPOList.stream()
                        .map(CrossLaneLightsPO::getLaneId)
                        .collect(Collectors.toList());
                if (CollectionUtil.isNotEmpty(laneIds)) {
                    List<LaneInfoPO> laneInfoPOList = laneInfoMapper.selectByLaneIds(laneIds);
                    // 根据转向去重
                    TreeSet<LaneInfoPO> laneInfoPOTreeSet = laneInfoPOList.stream()
                            .collect(Collectors.toCollection(
                                    () -> new TreeSet<>(Comparator.comparing(LaneInfoPO::getTurn))));
                    for (LaneInfoPO laneInfoPO : laneInfoPOTreeSet) {
                        SaveSchemeConfigDTO.LaneListElement laneListElement = new SaveSchemeConfigDTO.LaneListElement();
                        laneListElement.setDirection(laneInfoPO.getTurn());
                        laneListElement.setLaneType(laneInfoPO.getCategory());

                        laneList.add(laneListElement);
                    }
                }
            }
        }
        return laneList;
    }

    private boolean hasLightsId(Integer lightsId) {
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsMapper.selectByLightsId(lightsId);
        if (crossPhaseLightsPOList == null || crossPhaseLightsPOList.size() == 0) {
            return false;
        }
        return true;
    }

    private void handleRingTime(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        // 更新方案基础信息表
        updateCrossScheme(saveSchemeConfigDTO);
        // 更新相位基础表、相位灯组关系表、伴随相位母相位关系
        updateCrossPhase(saveSchemeConfigDTO);
        // 更新协调相位ID
        updateCoordPhase(saveSchemeConfigDTO);
    }

    private void updateCoordPhase(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            String schemeNo = phaseScheme.getSchemeNo();
            Integer schemeId = crossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
            String coordPhaseNo = phaseScheme.getCoordPhaseNo();
            Integer phaseId = crossPhaseMapper.selectIdByCrossIdAndCoordPhaseNo(crossId, schemeId, coordPhaseNo);
            if (phaseId != null) {
                crossSchemeMapper.updateCoordPhase(crossId, schemeNo, phaseId);
            }
        }
    }

    private void updateCrossPhase(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        crossPhaseMapper.deleteByCrossId(crossId);
        crossPhaseLightsMapper.deleteByCrossId(crossId);
        crossAccompanyPhaseMapper.deleteByCrossId(crossId);

        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            // 用于限制相位号不能重复
            Set<String> phaseNoSet = new HashSet<>();
            String schemeNo = phaseScheme.getSchemeNo();
            // 获取方案ID
            Integer schemeId = crossSchemeMapper.selectIdByCrossIdAndSchemeNo(crossId, schemeNo);
            List<SaveSchemeConfigDTO.RingListElement> ringList = phaseScheme.getRingList();
            // 用于维护伴随相位
            Map<String, SaveSchemeConfigDTO.PhaseListElement> phaseNoMap = new HashMap<>();
            for (SaveSchemeConfigDTO.RingListElement ringListElement : ringList) {
                Integer ringNo = ringListElement.getRingNo();
                List<SaveSchemeConfigDTO.GroupListElement> groupList = ringListElement.getGroupList();
                for (int i = 0; i < groupList.size(); i++) {
                    Integer groupNo = i + 1;
                    SaveSchemeConfigDTO.GroupListElement groupListElement = groupList.get(i);
                    List<SaveSchemeConfigDTO.PhaseListElement> phaseList = groupListElement.getPhaseList();
                    for (int j = 0; j < phaseList.size(); j++) {
                        SaveSchemeConfigDTO.PhaseListElement phase = phaseList.get(j);
                        String phaseNo = phase.getPhaseNo();
                        try {
                            Integer.parseInt(phaseNo);
                        } catch (Exception e) {
                            throw new RuntimeException("相位编号必须为数字");
                        }
                        testPhaseNo(phaseNoSet, phaseNo);
                        Integer phaseType = phase.getPhaseType();
                        if (phaseType != 3) { // 实相位或虚相位
                            phaseNoMap.put(phaseNo, phase);
                            Integer yellowFlashTime = phase.getYellowFlashTime();
                            Integer redFlashTime = phase.getRedFlashTime();
                            Integer greenTime = phase.getGreenTime();
                            Integer greenFlashTime = phase.getGreenFlashTime();
                            Integer yellowTime = phase.getYellowTime();
                            Integer redTime = phase.getRedTime();
                            Integer minGreenTime = phase.getMinGreenTime();
                            Integer maxGreenTime = phase.getMaxGreenTime();

                            CrossPhasePO crossPhasePO = new CrossPhasePO();
                            crossPhasePO.setPhaseNo(phaseNo);
                            crossPhasePO.setPhaseType(phaseType);
                            crossPhasePO.setSort(j + 1);
                            crossPhasePO.setCrossId(crossId);
                            crossPhasePO.setPlanId(schemeId);
                            crossPhasePO.setRingNo(ringNo);
                            crossPhasePO.setGroupNo(groupNo);
                            // 控制模式
                            crossPhasePO.setControlMode(ControlModeConst.FIXED_CYCLE);

                            crossPhasePO.setGreenTime(greenTime);
                            crossPhasePO.setGreenFlashTime(greenFlashTime);
                            crossPhasePO.setYellowTime(yellowTime);
                            crossPhasePO.setRedTime(redTime);
                            crossPhasePO.setYellowFlashTime(yellowFlashTime);
                            crossPhasePO.setRedFlashTime(redFlashTime);
                            crossPhasePO.setMinGreenTime(minGreenTime);
                            crossPhasePO.setMaxGreenTime(maxGreenTime);
                            crossPhasePO.setPhaseTime(greenTime + greenFlashTime + yellowTime + redTime
                                    + yellowFlashTime + redFlashTime);
                            // 插入相位表记录，获取ID
                            crossPhaseMapper.insertOne(crossPhasePO);
                            Integer phaseId = crossPhasePO.getId();
                            if (phaseType == 1) { // 实相位
                                // 更新相位灯组关系表
                                updateCrossPhaseLights(crossId, phase, phaseId);
                            }
                        } else { // 伴随相位
                            CrossPhasePO crossPhasePO = new CrossPhasePO();
                            crossPhasePO.setPhaseNo(phaseNo);
                            crossPhasePO.setPhaseType(phaseType);
                            crossPhasePO.setSort(j + 1);
                            crossPhasePO.setCrossId(crossId);
                            crossPhasePO.setPlanId(schemeId);
                            crossPhasePO.setRingNo(ringNo);
                            crossPhasePO.setGroupNo(groupNo);
                            // 控制模式
                            crossPhasePO.setControlMode(ControlModeConst.FIXED_CYCLE);
                            Integer coordGreenTime = 0;
                            Integer coordGreenFlashTime = 0;
                            Integer coordYelloFlashTime = 0;
                            Integer coordRedFlashTime = 0;
                            Integer coordYellowTime = 0;
                            Integer coordRedTime = 0;
                            Integer coordMinGreenTime = 0;
                            Integer coordMaxGreenTime = 0;
                            List<String> superPhaseNoList = phase.getAccompanyPhaseNoList();
                            if (superPhaseNoList != null) {
                                for (String superPhaseNo : superPhaseNoList) {
                                    SaveSchemeConfigDTO.PhaseListElement phaseListElement = phaseNoMap.get(superPhaseNo);
                                    // 时长加总
                                    coordGreenTime += phaseListElement.getGreenTime();
                                    coordGreenFlashTime += phaseListElement.getGreenFlashTime();
                                    coordYelloFlashTime += phaseListElement.getYellowFlashTime();
                                    coordRedFlashTime += phaseListElement.getRedFlashTime();
                                    coordYellowTime += phaseListElement.getYellowTime();
                                    coordRedTime += phaseListElement.getRedTime();
                                    coordMinGreenTime += phaseListElement.getMinGreenTime();
                                    coordMaxGreenTime += phaseListElement.getMaxGreenTime();
                                }
                                crossPhasePO.setGreenTime(coordGreenTime);
                                crossPhasePO.setGreenFlashTime(coordGreenFlashTime);
                                crossPhasePO.setYellowFlashTime(coordYelloFlashTime);
                                crossPhasePO.setRedFlashTime(coordRedFlashTime);
                                crossPhasePO.setYellowTime(coordYellowTime);
                                crossPhasePO.setRedTime(coordRedTime);
                                crossPhasePO.setMinGreenTime(coordMinGreenTime);
                                crossPhasePO.setMaxGreenTime(coordMaxGreenTime);
                                crossPhasePO.setPhaseTime(coordGreenTime + coordGreenFlashTime + coordYelloFlashTime
                                        + coordRedFlashTime + coordYellowTime + coordRedTime);
                                // 插入相位表记录获取ID
                                crossPhaseMapper.insertOne(crossPhasePO);
                                Integer accompanyPhaseId = crossPhasePO.getId();

                                // 更新伴随相位母相位关系表
                                List<CrossAccompanyPhasePO> crossAccompanyPhasePOList = new ArrayList<>();
                                for (String superPhaseNo : superPhaseNoList) {
                                    CrossAccompanyPhasePO crossAccompanyPhasePO = new CrossAccompanyPhasePO();
                                    crossAccompanyPhasePO.setCrossId(crossId);
                                    crossAccompanyPhasePO.setPhaseId(accompanyPhaseId);
                                    Integer superPhaseId = crossPhaseMapper.selectIdByCrossIdAndCoordPhaseNo(
                                            crossId, schemeId, superPhaseNo);
                                    crossAccompanyPhasePO.setSuperId(superPhaseId);
                                    crossAccompanyPhasePOList.add(crossAccompanyPhasePO);
                                }
                                crossAccompanyPhaseMapper.insertBatch(crossAccompanyPhasePOList);
                                // 更新相位灯组关系表
                                updateCrossPhaseLights(crossId, phase, accompanyPhaseId);
                            }
                        }
                    }
                }
            }
            // 处理特殊控制
            List<SaveSchemeConfigDTO.PhaseListElement> specialControlModeList = phaseScheme.getSpecialControlModeList();
            if (specialControlModeList != null) {
                for (int i = 0; i < specialControlModeList.size(); i++) {
                    SaveSchemeConfigDTO.PhaseListElement phase = specialControlModeList.get(i);
                    String phaseNo = phase.getPhaseNo();
                    try {
                        Integer.parseInt(phaseNo);
                    } catch (Exception e) {
                        throw new RuntimeException("相位编号必须为数字");
                    }
                    testPhaseNo(phaseNoSet, phaseNo);
                    CrossPhasePO crossPhasePO = new CrossPhasePO();
                    crossPhasePO.setPhaseNo(phaseNo);
                    crossPhasePO.setSort(i + 1);
                    crossPhasePO.setCrossId(crossId);
                    crossPhasePO.setPlanId(schemeId);
                    crossPhasePO.setPhaseType(1);
                    // 控制模式
                    if (phase.getYellowFlash() == 1) {
                        crossPhasePO.setControlMode(7);
                    } else if (phase.getIsRed() == 1) {
                        crossPhasePO.setControlMode(4);
                    } else if (phase.getIsGreen() == 1) {
                        crossPhasePO.setControlMode(2);
                    } else if (phase.getGreenFlash() == 1) {
                        crossPhasePO.setControlMode(6);
                    }
                    // 插入相位表记录，获取ID
                    crossPhaseMapper.insertOne(crossPhasePO);
                    Integer phaseId = crossPhasePO.getId();
                    // 更新相位灯组关系表
                    updateCrossPhaseLights(crossId, phase, phaseId);
                }
            }
        }
    }

    private static void testPhaseNo(Set<String> phaseNoSet, String phaseNo) throws PhaseNoNotUniqueException {
        boolean isUnique = phaseNoSet.add(phaseNo);
        if (!isUnique) {
            throw new PhaseNoNotUniqueException("同一方案中相位号不可重复");
        }
    }

    private void updateCrossPhaseLights(String crossId,
                                        SaveSchemeConfigDTO.PhaseListElement phase,
                                        Integer phaseId) {
        Set<CrossPhaseLightsPO> crossPhaseLightsPOSet = new HashSet<>();
        List<SaveSchemeConfigDTO.DirListElement> dirList = phase.getDirList();
        for (SaveSchemeConfigDTO.DirListElement dirListElement : dirList) {
            Integer dir = dirListElement.getDir();
            // 更新行人相位-灯组关系
            Integer hasPersonPhase = dirListElement.getHasPersonPhase();
            if (hasPersonPhase == 1) {
                List<CrossLightsPO> crossLightsPOList = crossLightsMapper.selectByCrossIdAndDir(crossId, dir);
                for (CrossLightsPO crossLightsPO : crossLightsPOList) {
                    Integer type = crossLightsPO.getType();
                    if (type >= 20 && type < 30) { // 行人灯
                        Integer lightId = crossLightsPO.getId();
                        CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                        crossPhaseLightsPO.setCrossId(crossId);
                        crossPhaseLightsPO.setLightsId(lightId);
                        crossPhaseLightsPO.setPhaseId(phaseId);
                        crossPhaseLightsPOSet.add(crossPhaseLightsPO);
                    }
                }
            }
            List<SaveSchemeConfigDTO.LaneListElement> laneList = dirListElement.getLaneList();
            for (SaveSchemeConfigDTO.LaneListElement laneListElement : laneList) {
                CrossPhaseLightsPO crossPhaseLightsPO = new CrossPhaseLightsPO();
                Integer category = laneListElement.getLaneType();
                Integer turn = laneListElement.getDirection();
                // 根据车道获得灯组ID
                String laneId = laneInfoMapper.selectIdByCrossIdDirTurn(crossId, dir, turn, category);
                Integer lightsId = crossLaneLightsMapper.selectLightsIdByLaneId(crossId, laneId);
                if (lightsId != null) {
                    crossPhaseLightsPO.setLightsId(lightsId);
                }
                crossPhaseLightsPO.setPhaseId(phaseId);
                crossPhaseLightsPO.setCrossId(crossId);
                crossPhaseLightsPOSet.add(crossPhaseLightsPO);
            }
        }
        crossPhaseLightsMapper.insertBatch(crossPhaseLightsPOSet);
    }

    private void updateCrossScheme(SaveSchemeConfigDTO saveSchemeConfigDTO) {
        String crossId = saveSchemeConfigDTO.getCrossId();
        List<SaveSchemeConfigDTO.PhaseScheme> phaseSchemeList = saveSchemeConfigDTO.getPhaseSchemeList();
        // 删除数据口里有，但是DTO里没有的方案ID对应的记录
        Set<Integer> existedIds = crossSchemeMapper.selectIdsByCrossId(crossId);
        if (CollectionUtil.isNotEmpty(existedIds)) {
            Set<Integer> dtoIds = phaseSchemeList.stream()
                    .map(SaveSchemeConfigDTO.PhaseScheme::getSchemeId)
                    .collect(Collectors.toSet());
            if (CollectionUtil.isNotEmpty(dtoIds)) {
                existedIds.removeAll(dtoIds);
                if (CollectionUtil.isNotEmpty(existedIds)){
                    crossSchemeMapper.deleteByIds(existedIds);
                }
            }
        }
        for (SaveSchemeConfigDTO.PhaseScheme phaseScheme : phaseSchemeList) {
            // 构造PO
            CrossSchemePO crossSchemePO = new CrossSchemePO();
            String schemeNo = phaseScheme.getSchemeNo();// 方案号
            crossSchemePO.setSchemeNo(schemeNo);
            String name = phaseScheme.getName(); // 方案名称
            crossSchemePO.setName(name);
            crossSchemePO.setCrossId(crossId);
            crossSchemePO.setCycle(phaseScheme.getCycle());
            Integer offset = phaseScheme.getOffset();
            if (offset != null) {
                crossSchemePO.setOffset(offset);
            }
            crossSchemePO.setSource(2); // 来自平台
            crossSchemePO.setIsDeleted(0);
            // 新增或修改数据
            Integer schemeId = phaseScheme.getSchemeId();
            if (ObjectUtil.isEmpty(schemeId)){
                // 没有方案ID视为新增
                crossSchemeMapper.insertOne(crossSchemePO);
            } else {
                // 有方案ID视为修改
                crossSchemePO.setId(schemeId);
                crossSchemeMapper.update(crossSchemePO);
            }
        }
    }
}
